﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

using Quartz;
using SCC.Common;
using SCC.Models;
using SCC.Interface;
using Newtonsoft.Json;
using HtmlAgilityPack;

namespace SCC.Crawler.DT
{
    /// <summary>
    /// 数据爬取类
    /// 福建36选7
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class FJ36X7Job : IJob
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        public FJ36X7Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        /// 作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            //预设节假日不开奖
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"))) return;
            LatestItem = context.JobDetail.JobDataMap["LatestItem"] as OpenCode8DTModel;
            try
            {
                //服务启动时配置初始数据
                if (LatestItem == null)
                {
                    LatestItem = services.GetOpenCode8DTLastItem(currentLottery);
                    if (LatestItem == null)
                    {
                        //服务第一次启动，数据库一条数据都没有时
                        LatestItem = new OpenCode8DTModel()
                        {
                            Term = CommonHelper.GenerateQiHaoYYQQQ(0),
                            OpenTime = new DateTime(CommonHelper.SCCSysDateTime.Year, 1, 1)
                        };
                    }
                }
                //程序时间第二天，程序根据配置检查是否昨天有开奖
                isGetData = false;
                if (CommonHelper.CheckDTIsNeedGetData(Config))
                {
                    CheckingOpenDayTheLotteryData();
                }
                if (!LatestItem.Term.ToString().StartsWith(CommonHelper.SCCSysDateTime.ToString("yy")))
                {
                    //翻年时
                    LatestItem = new OpenCode8DTModel()
                    {
                        Term = CommonHelper.GenerateQiHaoYYQQQ(0),
                        OpenTime = new DateTime(CommonHelper.SCCSysDateTime.Year, 1, 1)
                    };
                }
                //当今日开奖并且当前时间是晚上8点过后开始抓取
                if (CommonHelper.CheckTodayIsOpenDay(Config) && CommonHelper.SCCSysDateTime.Hour > 12)
                {

                    DoTodayJobByDB();
                    //DoTodayJobByMainUrl();
                    //DoTodayJobByBackUrl();
                }
                if (isGetData)
                    TrendChartHelper.GenerateFJ36X7TrendChart(log);
            }
            catch (Exception ex)
            {
                log.Error(typeof(FJ36X7Job), string.Format("【{0}】抓取时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            //保存最新期号
            context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
        }
        /// <summary>
        /// 通过数据库获取开奖数据
        /// </summary>
        private void DoTodayJobByDB()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl))
            {
                var OpenList = services.GetYesterdayFailQQListDB7_toDay(currentLottery, Config.TimesPerDay);
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.OrderByDescending(a=>a.Key).First().Key.ToString();
                var startQiNum = Convert.ToInt32(LatestItem.Term.ToString().Substring(2)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(2));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                string getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = LatestItem.Term.ToString().Substring(0, 2) + i.ToString().PadLeft(3, '0');
                    var matchItemq = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItemq.Key != null)
                    {
                        var opencodeNode = matchItemq.Value.Split(',');
                        OpenCode8DTModel matchItem = new OpenCode8DTModel();
                        matchItem.OpenTime = Convert.ToDateTime(DateTime.Now);
                        matchItem.Addtime = Convert.ToDateTime(DateTime.Now);
                        matchItem.Term = Convert.ToInt64(matchItemq.Key);
                        matchItem.OpenCode1 = Convert.ToInt32(opencodeNode[0]);
                        matchItem.OpenCode2 = Convert.ToInt32(opencodeNode[1]);
                        matchItem.OpenCode3 = Convert.ToInt32(opencodeNode[2]);
                        matchItem.OpenCode4 = Convert.ToInt32(opencodeNode[3]);
                        matchItem.OpenCode5 = Convert.ToInt32(opencodeNode[4]);
                        matchItem.OpenCode6 = Convert.ToInt32(opencodeNode[5]);
                        matchItem.OpenCode7 = Convert.ToInt32(opencodeNode[6]);
                        matchItem.OpenCode8 = Convert.ToInt32(opencodeNode[7]);
                        matchItem.Detail = Convert.ToString(opencodeNode[9]);
                        if (matchItemq.Key != null && services.AddDTOpen8Code(currentLottery, matchItem))
                        {
                            //Do Success Log
                            log.Info(typeof(FJ36X7Job), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                            LatestItem = matchItem;
                            isGetData = true;
                        }
                    }
                    
                }
            }
        }

        /// <summary>
        /// 自检爬取未爬取到的开奖数据，并对昨日开奖但未爬取到开奖数据的彩种添加邮件提醒
        /// </summary>
        private void CheckingOpenDayTheLotteryData()
        {
            //从数据库中获取昨天数据抓取失败列表
            FailedQiHaoList = services.GetFailedYYQQQList(currentLottery);
            if (FailedQiHaoList.Count > 0)
            {
                DoYesterdayFailedListByMainUrl();
                DoYesterdayFailedListByBackUrl();
                foreach (var fQiHao in FailedQiHaoList)
                {
                    //将抓取失败数据推送至邮件列表，待邮件服务发送至配置管理员的邮箱中
                    if (email.AddEmail(Config.Area + Config.LotteryName, fQiHao, CommonHelper.GenerateDTOpenTime(Config)))
                        log.Error(typeof(FJ36X7Job), CommonHelper.GetJobLogError(Config, fQiHao));
                }
            }
            if (LatestItem.OpenTime.ToString("yyyyMMdd") != CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyyMMdd"))
            {
                //开奖时间(昨天)未抓取到最新开奖数据,则再抓取一次，若还不成功则写入待发送邮件列表
                DoTodayJobByMainUrl();
                DoTodayJobByBackUrl();
                if (LatestItem.OpenTime.ToString("yyyyMMdd") != CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyyMMdd"))
                {
                    var openQiHao = (LatestItem.Term + 1).ToString();
                    if (email.AddEmail(Config.Area + Config.LotteryName, openQiHao, CommonHelper.GenerateDTOpenTime(Config)))
                        log.Error(typeof(FJ36X7Job), CommonHelper.GetJobLogError(Config, openQiHao));
                }
            }
        }

        /// <summary>
        /// 通过主站点爬取开奖数据
        /// (百度乐彩)
        /// </summary>
        private void DoTodayJobByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl))
            {
                var OpenList = GetOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.First().Term.ToString();
                var startQiNum = Convert.ToInt32(LatestItem.Term.ToString().Substring(2)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(2));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                var getQiHao = string.Empty;
                OpenCode8DTModel matchItem = null;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = LatestItem.Term.ToString().Substring(0, 2) + i.ToString().PadLeft(3, '0');
                    matchItem = OpenList.Where(R => R.Term.ToString() == getQiHao).FirstOrDefault();
                    if (matchItem != null && OptimizeMainModel(ref matchItem) && services.AddDTOpen8Code(currentLottery, matchItem))
                    {
                        //Do Success Log
                        log.Info(typeof(FJ36X7Job), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                        LatestItem = matchItem;
                        isGetData = true;
                    }
                }
            }
        }

        /// <summary>
        /// 通过主站爬取错误期号列表中每一个期号
        /// (百度乐彩)
        /// </summary>
        private void DoYesterdayFailedListByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetOpenListFromMainUrl_New();
                if (OpenList.Count == 0) return;//无抓取数据
                OpenCode8DTModel matchItem = null;
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    matchItem = OpenList.Where(R => R.Term.ToString() == failedQiHao).FirstOrDefault();
                    if (matchItem != null && OptimizeMainModel(ref matchItem) && services.AddDTOpen8Code(currentLottery, matchItem))
                    {
                        //Do Success Log
                        log.Info(typeof(FJ36X7Job), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                        if (matchItem.Term > LatestItem.Term)
                        {
                            LatestItem = matchItem;
                        }
                        SuccessList.Add(failedQiHao);
                        isGetData = true;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }
        /// <summary>
        /// 获取主站开奖列表数据
        /// </summary>
        /// <returns></returns>
        private List<OpenCode8DTModel> GetOpenListFromMainUrl()
        {
            List<OpenCode8DTModel> result = new List<OpenCode8DTModel>();
            try
            {
                string requestUrl = string.Format("{0}?r={1}", Config.MainUrl, new Random().Next(1000, 9999));
                var htmlResource = NetHelper.GetBaiDuLeCaiResponse(requestUrl);
                if (string.IsNullOrWhiteSpace(htmlResource)) return result;
                HtmlDocument doc = new HtmlDocument();
                doc.LoadHtml(htmlResource);
                var table = doc.DocumentNode.SelectSingleNode("//tbody");
                if (table == null) return result;
                var trs = table.ChildNodes.Where(node => node.Name == "tr").ToList();
                OpenCode8DTModel model = null;
                for (var i = 0; i < trs.Count; i++)//第一行
                {
                    var trstyle = trs[i].Attributes["style"];
                    if (trstyle != null && trstyle.Value == "display:none")
                    {
                        continue;
                    }
                    var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();
                    if (tds.Count < 4) continue;
                    model = new OpenCode8DTModel();
                    model.OpenTime = Convert.ToDateTime(tds[0].InnerText);
                    model.Term = Convert.ToInt64(tds[1].ChildNodes.Where(n => n.Name == "a").ToList()[0].InnerText.Trim());
                    if (tds[2].ChildNodes.Count == 0) continue;
                    var opencodeNode = tds[2].ChildNodes[0].ChildNodes.Where(n => n.Name == "span").ToList();
                    if (opencodeNode.Count < 8) continue;
                    model.OpenCode1 = Convert.ToInt32(opencodeNode[0].InnerText.Trim());
                    model.OpenCode2 = Convert.ToInt32(opencodeNode[1].InnerText.Trim());
                    model.OpenCode3 = Convert.ToInt32(opencodeNode[2].InnerText.Trim());
                    model.OpenCode4 = Convert.ToInt32(opencodeNode[3].InnerText.Trim());
                    model.OpenCode5 = Convert.ToInt32(opencodeNode[4].InnerText.Trim());
                    model.OpenCode6 = Convert.ToInt32(opencodeNode[5].InnerText.Trim());
                    model.OpenCode7 = Convert.ToInt32(opencodeNode[6].InnerText.Trim());
                    model.OpenCode8 = Convert.ToInt32(opencodeNode[7].InnerText.Trim());
                    result.Add(model);
                }
                result = result.OrderByDescending(S => S.Term).ToList();
            }
            catch (Exception ex)
            {
                log.Error(typeof(FJ36X7Job), string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 获取数据库表数据
        /// </summary>
        /// <returns></returns>
        private List<OpenCode8DTModel> GetOpenListFromMainUrl_New()
        {
            List<OpenCode8DTModel> result = new List<OpenCode8DTModel>();
            try
            {

                var list = services.GetYesterdayFailQQListDB7(currentLottery, Config.TimesPerDay);

                foreach (var item in list)
                {
                    var opencodeNode = item.Value.Split(',');
                    OpenCode8DTModel model = new OpenCode8DTModel();
                    model.OpenTime = Convert.ToDateTime(opencodeNode[8]);
                    model.Term = Convert.ToInt64(item.Key);
                    model.OpenCode1 = Convert.ToInt32(opencodeNode[0]);
                    model.OpenCode2 = Convert.ToInt32(opencodeNode[1]);
                    model.OpenCode3 = Convert.ToInt32(opencodeNode[2]);
                    model.OpenCode4 = Convert.ToInt32(opencodeNode[3]);
                    model.OpenCode5 = Convert.ToInt32(opencodeNode[4]);
                    model.OpenCode6 = Convert.ToInt32(opencodeNode[5]);
                    model.OpenCode7 = Convert.ToInt32(opencodeNode[6]);
                    model.OpenCode8 = Convert.ToInt32(opencodeNode[7]);
                    result.Add(model);

                }

                 
               
                result = result.OrderByDescending(S => S.Term).ToList();
            }
            catch (Exception ex)
            {
                log.Error(typeof(FJ36X7Job), string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 完善备用站点江苏体彩7位数开奖实体信息
        /// </summary>
        /// <param name="model"></param>
        private bool OptimizeMainModel(ref OpenCode8DTModel model)
        {
            var url = string.Format("http://baidu.lecai.com/lottery/draw/view/509/{0}?r={1}", model.Term, new Random().Next(1000, 9999));
            try
            {
                var htmlResource = NetHelper.GetBaiDuLeCaiResponse(url);
                Regex reg = new Regex(@"var phaseData = ([\s\S]*?);");
                Match m = reg.Match(htmlResource);
                if (m.Success)
                {
                    var dataJson = m.Result("$1");
                    var obj = JsonConvert.DeserializeObject<dynamic>(dataJson);
                    var data = obj[model.Term.ToString()];
                    if (data != null)
                    {
                        int Level1Num = 0, Level2Num = 0, Level3Num = 0, Level4Num = 0, Level5Num = 0, Level6Num = 0, Level7Num = 0, Level8Num = 0;
                        decimal Level1Money = 0, Level2Money = 0, Level3Money = 0, Level4Money = 0, Level5Money = 0, Level6Money = 0, Level7Money = 0, Level8Money = 0, Sales = 0, Jackpot = 0;
                        Jackpot = Convert.ToDecimal(data["formatPoolAmount"]);
                        Sales = Convert.ToDecimal(data["formatSaleAmount"]);
                        Level1Num = Convert.ToInt32(data["list"]["prize1"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level1Money = Convert.ToDecimal(data["list"]["prize1"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        Level2Num = Convert.ToInt32(data["list"]["prize2"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level2Money = Convert.ToDecimal(data["list"]["prize2"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        Level3Num = Convert.ToInt32(data["list"]["prize3"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level3Money = Convert.ToDecimal(data["list"]["prize3"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        Level4Num = Convert.ToInt32(data["list"]["prize4"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level4Money = Convert.ToDecimal(data["list"]["prize4"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        Level5Num = Convert.ToInt32(data["list"]["prize5"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level5Money = Convert.ToDecimal(data["list"]["prize5"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        Level6Num = Convert.ToInt32(data["list"]["prize6"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level6Money = Convert.ToDecimal(data["list"]["prize6"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        Level7Num = Convert.ToInt32(data["list"]["prize7"]["bet"].Value.Replace(",", string.Empty).Replace("注", string.Empty));
                        Level7Money = Convert.ToDecimal(data["list"]["prize7"]["prize"].Value.Replace(",", string.Empty).Replace("元", string.Empty));
                        model.Detail = string.Format("{0},{1}^特等奖|{2}|{3},一等奖|{4}|{5},二等奖|{6}|{7},三等奖|{8}|{9},四等奖|{10}|{11},五等奖|{12}|{13},六等奖|{14}|{15},幸运奖|{16}|{17}",
                            Sales, Jackpot, Level1Num, Level1Money, Level2Num, Level2Money, Level3Num, Level3Money, Level4Num,
                            Level4Money, Level5Num, Level5Money, Level6Num, Level6Money, Level7Num, Level7Money, Level8Num, Level8Money);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(FJ36X7Job), string.Format("【{0}】通过主站点优化开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return false;
        }

        /// <summary>
        /// 通过备用站点抓取开奖数据
        /// (福建体彩网)
        /// </summary>
        private void DoTodayJobByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl))
            {
                var OpenList = GetOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.First().Term.ToString();
                var startQiNum = Convert.ToInt32(LatestItem.Term.ToString().Substring(2)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(2));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                string getQiHao = string.Empty;
                OpenCode8DTModel matchItem = null;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = LatestItem.Term.ToString().Substring(0, 2) + i.ToString().PadLeft(3, '0');
                    matchItem = OpenList.Where(R => R.Term.ToString() == getQiHao).FirstOrDefault();
                    if (matchItem != null && services.AddDTOpen8Code(currentLottery, matchItem))
                    {
                        //Do Success Log
                        log.Info(typeof(FJ36X7Job), CommonHelper.GetJobBackLogInfo(Config, getQiHao));
                        LatestItem = matchItem;
                        isGetData = true;
                    }
                }
            }
        }

        /// <summary>
        /// 通过备用地址抓取错误期号列表中每一个期号
        /// (福建体彩网)
        /// </summary>
        private void DoYesterdayFailedListByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                OpenCode8DTModel matchItem = null;
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    matchItem = OpenList.Where(R => R.Term.ToString() == failedQiHao).FirstOrDefault();
                    if (matchItem != null && services.AddDTOpen8Code(currentLottery, matchItem))
                    {
                        //Do Success Log
                        log.Info(typeof(FJ36X7Job), CommonHelper.GetJobBackLogInfo(Config, failedQiHao));
                        if (matchItem.Term > LatestItem.Term)
                        {
                            LatestItem = matchItem;
                        }
                        isGetData = true;
                        SuccessList.Add(failedQiHao);
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }

        /// <summary>
        /// 获取备用站点开奖列表数据
        /// </summary>
        /// <returns></returns>
        private List<OpenCode8DTModel> GetOpenListFromBackUrl()
        {
            List<OpenCode8DTModel> result = new List<OpenCode8DTModel>();
            try
            {
                var htmlResource = NetHelper.GetUrlResponse(Config.BackUrl + string.Format("&r={0}", new Random().Next(1000, 9999)));//只取第一页数据，最新5条记录
                if (!string.IsNullOrWhiteSpace(htmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(htmlResource);
                    var divSections = doc.DocumentNode.SelectNodes("//div[@class='artCon KJDetail']").ToList();
                    OpenCode8DTModel model = null;
                    var lastYear = (CommonHelper.SCCSysDateTime.Year - 1).ToString().Substring(2);
                    int Level1Num = 0, Level2Num = 0, Level3Num = 0, Level4Num = 0, Level5Num = 0, Level6Num = 0, Level7Num = 0, Level8Num = 0;
                    decimal Level1Money = 0, Level2Money = 0, Level3Money = 0, Level4Money = 0, Level5Money = 0, Level6Money = 0, Level7Money = 0, Level8Money = 0, Sales = 0, Jackpot = 0;
                    HtmlNode divSummary, divDetail, table, tbody;
                    List<HtmlNode> divContent, divInfos, spans, trs, tds, divDetails, divTexts;
                    string term = string.Empty, detailName = string.Empty, detailNum = string.Empty, detailMoney = string.Empty;
                    string[] openCode = null;
                    Regex reg = null;
                    Match m = null;
                    foreach (var divSection in divSections)
                    {
                        model = new OpenCode8DTModel();
                        divInfos = divSection.ChildNodes.Where(N => N.Name.ToLower() == "div").ToList();
                        if (divInfos.Count < 2) continue;
                        divContent = divInfos[1].ChildNodes.Where(N => N.Name.ToLower() == "div").ToList();
                        if (divContent.Count < 2) continue;
                        divSummary = divContent[0];//概述
                        divDetail = divContent[1];//详情
                        spans = divSummary.ChildNodes.Where(N => N.Name.ToLower() == "strong").First().ChildNodes.Where(N => N.Name.ToLower() == "span").ToList();
                        term = spans[0].InnerText;
                        if (term.StartsWith(lastYear))
                        {
                            break;
                        }
                        model.Term = Convert.ToInt32(term);
                        model.OpenTime = Convert.ToDateTime(spans[2].InnerText.Replace("年", "-").Replace("月", "-").Replace("日", string.Empty));
                        openCode = (spans[3].InnerText + " " + spans[4].InnerText).Trim().Split(' ');
                        model.OpenCode1 = Convert.ToInt32(openCode[0]);
                        model.OpenCode2 = Convert.ToInt32(openCode[1]);
                        model.OpenCode3 = Convert.ToInt32(openCode[2]);
                        model.OpenCode4 = Convert.ToInt32(openCode[3]);
                        model.OpenCode5 = Convert.ToInt32(openCode[4]);
                        model.OpenCode6 = Convert.ToInt32(openCode[5]);
                        model.OpenCode7 = Convert.ToInt32(openCode[6]);
                        model.OpenCode8 = Convert.ToInt32(openCode[7]);
                        Sales = Convert.ToDecimal(spans[1].InnerText);
                        table = divDetail.ChildNodes.Where(N => N.Name.ToLower() == "table").FirstOrDefault();
                        if (table == null)
                        {
                            divDetails = divDetail.ChildNodes.Where(N => N.Name.ToLower() == "div").ToList();
                            if (divDetails.Count < 14) continue;
                            for (var i = 4; i < 12; i++)
                            {
                                divTexts = divDetails[i].ChildNodes.Where(N => N.Name.ToLower() == "#text").ToList();
                                if (divTexts.Count < 4) continue;
                                detailName = divTexts[0].InnerText.Replace("&nbsp;", string.Empty).Trim();
                                detailNum = divTexts[1].InnerText.Replace("&nbsp;", string.Empty).Replace("注", string.Empty).Trim();
                                detailMoney = divTexts[2].InnerText.Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Replace("元", string.Empty).Trim();
                                if (detailName == "特等奖")
                                {
                                    Level1Num = Convert.ToInt32(detailNum);
                                    Level1Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "一等奖")
                                {
                                    Level2Num = Convert.ToInt32(detailNum);
                                    Level2Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "二等奖")
                                {
                                    Level3Num = Convert.ToInt32(detailNum);
                                    Level3Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "三等奖")
                                {
                                    Level4Num = Convert.ToInt32(detailNum);
                                    Level4Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "四等奖")
                                {
                                    Level5Num = Convert.ToInt32(detailNum);
                                    Level5Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "五等奖")
                                {
                                    Level6Num = Convert.ToInt32(detailNum);
                                    Level6Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "六等奖")
                                {
                                    Level7Num = Convert.ToInt32(detailNum);
                                    Level7Money = Convert.ToDecimal(detailMoney);
                                }
                                else if (detailName == "幸运奖")
                                {
                                    Level8Num = Convert.ToInt32(detailNum);
                                    Level8Money = Convert.ToDecimal(detailMoney);
                                }
                            }
                        }
                        else
                        {
                            tbody = table.ChildNodes.Where(N => N.Name.ToLower() == "tbody").FirstOrDefault();
                            if (tbody == null) continue;
                            trs = tbody.ChildNodes.Where(N => N.Name.ToLower() == "tr").ToList();
                            for (var i = 4; i < trs.Count; i++)//第一二三四行是表头
                            {
                                tds = trs[i].ChildNodes.Where(N => N.Name.ToLower() == "td").ToList();
                                if (tds.Count < 4) continue;
                                if (tds[0].InnerText.Trim() == "特等奖")
                                {
                                    Level1Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level1Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "一等奖")
                                {
                                    Level2Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level2Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "二等奖")
                                {
                                    Level3Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level3Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "三等奖")
                                {
                                    Level4Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level4Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "四等奖")
                                {
                                    Level5Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level5Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "五等奖")
                                {
                                    Level6Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level6Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "六等奖")
                                {
                                    Level7Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level7Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                                else if (tds[0].InnerText.Trim() == "幸运奖")
                                {
                                    Level8Num = Convert.ToInt32(tds[1].InnerText.Replace("注", string.Empty).Replace("&nbsp;", string.Empty).Replace(",", string.Empty).Trim());
                                    Level8Money = Convert.ToDecimal(tds[2].InnerText.Replace("元", string.Empty).Replace("&nbsp;", string.Empty).Trim());
                                }
                            }
                        }
                        //抓取奖池累计数据
                        reg = new Regex(@"([\d,.]*?)元奖金滚入下期奖池");
                        m = reg.Match(divSection.InnerHtml);
                        if (m.Success)
                            Jackpot = Convert.ToDecimal(m.Result("$1"));
                        model.Detail = string.Format("{0},{1}^特等奖|{2}|{3},一等奖|{4}|{5},二等奖|{6}|{7},三等奖|{8}|{9},四等奖|{10}|{11},五等奖|{12}|{13},六等奖|{14}|{15},幸运奖|{16}|{17}",
                            Sales, Jackpot, Level1Num, Level1Money, Level2Num, Level2Money, Level3Num, Level3Money, Level4Num,
                            Level4Money, Level5Num, Level5Money, Level6Num, Level6Money, Level7Num, Level7Money, Level8Num, Level8Money);
                        result.Add(model);
                    }
                }
                result = result.OrderByDescending(S => S.Term).ToList();
            }
            catch (Exception ex)
            {
                log.Error(typeof(FJ36X7Job), string.Format("【{0}】通过备用站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        #region Attribute
        /// <summary>
        /// 配置信息
        /// </summary>
        private SCCConfig Config = null;
        /// <summary>
        /// 当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode8DTModel LatestItem = null;
        /// <summary>
        /// 当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;
        /// <summary>
        /// 日志对象
        /// </summary>
        private LogHelper log = null;
        /// <summary>
        /// 数据服务
        /// </summary>
        private IDTOpenCode services = null;
        /// <summary>
        /// 当前彩种
        /// </summary>
        private SCCLottery currentLottery
        {
            get
            {
                return SCCLottery.FJ36X7;
            }
        }
        /// <summary>
        /// 邮件接口
        /// </summary>
        private IEmail email = null;
        /// <summary>
        /// 是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData = false;
        #endregion
    }
}
