﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Quartz;
using SCC.Common;
using SCC.Models;
using SCC.Interface;
using HtmlAgilityPack;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace SCC.Crawler.GP
{
    /// <summary>
    /// 数据爬取类
    /// 湖南快乐十分
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class HUNKL10FJob : IJob
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        public HUNKL10FJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IOpen8Code>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        /// 作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            //预设节假日不开奖
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"))) return;
            LatestQiHao = context.JobDetail.JobDataMap.GetString("LatestQiHao");
            try
            {
                //服务启动时配置初始数据
                if (string.IsNullOrEmpty(LatestQiHao))
                {
                    var lastItem = services.GetLastItem(currentLottery);
                    if (lastItem != null)
                    {
                        LatestQiHao = lastItem.Term.ToString();
                    }
                }
                //第一次启动服务或最新期号为昨天的开奖期号，则自检昨天开奖数据是否抓取完毕(否则插入邮件数据)，并重置当天期号和失败列表
                if (string.IsNullOrEmpty(LatestQiHao) || !LatestQiHao.StartsWith(CommonHelper.SCCSysDateTime.ToString("yyMMdd")))
                {
                    CheckingYesterdayTheLotteryData();
                    LatestQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQQ(0);
                }
                //当最新期号不符合当天总期数，执行当天作业
                if (Convert.ToInt32(LatestQiHao.Substring(6)) != Config.TimesPerDay)
                {
                    DoTodayJobByMainUrl();
                    DoTodayJobByBackUrl();
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(HUNKL10FJob), string.Format("【{0}】抓取时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            //保存最新期号和失败期号列表
            context.JobDetail.JobDataMap["LatestQiHao"] = LatestQiHao;
        }

        /// <summary>
        /// 自检昨天开奖数据
        /// </summary>
        private void CheckingYesterdayTheLotteryData()
        {
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyyMMdd"))) return;//如果昨日设定不开奖则不自检昨日开奖数据
            //从数据库中获取昨天数据抓取失败列表
            FailedQiHaoList = services.GetYesterdayFailQQQList(currentLottery, Config.TimesPerDay);
            if (FailedQiHaoList.Count > 0)
            {
                DoYesterdayFailedListByMainUrl();
                DoYesterdayFailedListByBackUrl();
                foreach (var fQiHao in FailedQiHaoList)
                {
                    //将抓取失败数据推送至邮件列表，待邮件服务发送至配置管理员的邮箱中
                    if (email.AddEmail(Config.Area + Config.LotteryName, fQiHao, CommonHelper.GenerateYesterdayOpenTime(Config, fQiHao)))
                        log.Error(typeof(HUNKL10FJob), CommonHelper.GetJobLogError(Config, fQiHao));
                }
            }
        }

        /// <summary>
        /// 通过主站点抓取开奖数据
        /// (湖南福彩官网)
        /// </summary>
        private void DoTodayJobByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl))
            {
                var OpenList = GetTodayOpenListFromMainUrl();

                if (OpenList.Count == 0) return;//无抓取数据

                //var newestQiHao = OpenList.First().Key;
                //var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                //var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                //if (startQiNum > newestQiNum) return;//无最新数据
                ////处理最新开奖数据
                //string getQiHao = string.Empty;
                //for (var i = startQiNum; i <= newestQiNum; i++)
                //{
                //    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQQ(i);
                //    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                //    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                //    {
                //        //处理成功写入日志
                //        log.Info(typeof(HUNKL10FJob), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                //        LatestQiHao = getQiHao;
                //    }
                //}

                foreach (KeyValuePair<string, string> pair in OpenList)
                {
                    string[] arr = pair.Key.Split("|".ToCharArray());

                    if (pair.Key != null && SaveRecord(pair.Key, pair.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(HUNKL10FJob), CommonHelper.GetJobMainLogInfo(Config, arr[0]));
                        LatestQiHao = arr[0];
                    }
                }
            }
        }

        /// <summary>
        /// 通过主站抓取错误期号列表中每一个期号
        /// (湖南福彩官网)
        /// </summary>
        private void DoYesterdayFailedListByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetTodayOpenListFromMainUrl(false);
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(HUNKL10FJob), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }

        /// <summary>
        /// 通过主站点抓取今日最新开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromMainUrl(bool IsToday = true)
        {
            //var requestUrl = string.Format("{0}&inqs={1}", Config.MainUrl, CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"));

            var day = DateTime.Now.ToString("yyyyMMdd");
            if (!IsToday) day = DateTime.Now.AddDays(-1).ToString("yyyyMMdd");
            var url = string.Format(Config.MainUrl, day);

            return GetOpenListFromMainUrl(url);
        }


        /// <summary>
        /// 抓取主站点开奖数据
        /// </summary>
        /// <param name="url">主站点</param>
        /// <returns></returns>
        private Dictionary<string, string> GetOpenListFromMainUrl(string url)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                #region 废弃代码 BY 大师兄
                //var requestUrl = string.Format("{0}&inqs={1}", url, CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"));
                //var HtmlResource = NetHelper.GetUrlResponse(requestUrl, Encoding.GetEncoding("gb2312"));
                //if (!string.IsNullOrWhiteSpace(HtmlResource))
                //{
                //    HtmlDocument doc = new HtmlDocument();
                //    doc.LoadHtml(HtmlResource);
                //    var tables = doc.DocumentNode.SelectNodes("//table");
                //    var target = tables.Where(T => T.Attributes["width"].Value == "98%").FirstOrDefault();
                //    if (target == null) return result;
                //    var trs = target.ChildNodes.Where(R => R.Name.ToLower() == "tr").ToList();
                //    if (trs.Count < 2) return result;
                //    List<HtmlNode> tds = null;
                //    string matchQiHao = string.Empty;
                //    string matchKJHaoMa = string.Empty;
                //    for (var i = 1; i < trs.Count; i++)
                //    {
                //        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                //        if (tds.Count < 3) continue;
                //        matchQiHao = CommonHelper.SCCSysDateTime.ToString("yy") + tds[1].InnerText.Trim();
                //        if (tds[2].ChildNodes[0] == null || tds[2].ChildNodes[0].ChildNodes[0] == null) continue;
                //        var childTds = tds[2].ChildNodes[0].ChildNodes[0].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                //        if (childTds.Count < 24) continue;
                //        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4},{5},{6},{7}", childTds[0].InnerText.Trim(), childTds[3].InnerText.Trim(), childTds[6].InnerText.Trim()
                //            , childTds[9].InnerText.Trim(), childTds[12].InnerText.Trim(), childTds[15].InnerText.Trim(), childTds[18].InnerText.Trim(), childTds[21].InnerText.Trim());
                //        if (!result.ContainsKey(matchQiHao))
                //            result.Add(matchQiHao, matchKJHaoMa);
                //    }
                //} 
                #endregion

                var HtmlResource = NetHelper.GetUrlResponse(url);
                if (HtmlResource == null) return null;

                var doc = new XmlDocument();
                doc.LoadXml(HtmlResource);
                var records = doc.SelectNodes("//row");
                if (records == null) return null;

                foreach (XmlNode xmlnode in records)
                {
                    if (xmlnode.Attributes["expect"] == null || string.IsNullOrWhiteSpace(xmlnode.Attributes["expect"].Value) || xmlnode.Attributes["opencode"] == null || string.IsNullOrWhiteSpace(xmlnode.Attributes["opencode"].Value)) continue;
                    var num = xmlnode.Attributes["expect"].Value.Trim();

                    //201 801 191 983
                    //180 319 001

                    var opencode = xmlnode.Attributes["opencode"].Value.Trim();
                    var opentime = xmlnode.Attributes["opentime"].Value.Trim();

                    result.Add(num + "|" + opentime, opencode);
                    //result.Add(num, opencode);
                    
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(HUNKL10FJob), string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        /// <summary>
        /// 通过备用站点抓取开奖数据
        /// (爱彩乐)
        /// </summary>
        private void DoTodayJobByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl))
            {
                var OpenList = GetTodayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.First().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                var getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(HUNKL10FJob), CommonHelper.GetJobBackLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过备用地址抓取错误期号列表中每一个期号
        /// (爱彩乐)
        /// </summary>
        private void DoYesterdayFailedListByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (var failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(HUNKL10FJob), CommonHelper.GetJobBackLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }
        /// <summary>
        /// 通过备用站点抓取今日最新开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromBackUrl()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.BackUrl);
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var table = doc.DocumentNode.SelectSingleNode("//table");
                    if (table == null) return result;
                    var trs = table.ChildNodes.Where(R => R.Name.ToLower() == "tr").ToList();
                    List<HtmlNode> tds = null, ems = null;
                    var matchQiHao = string.Empty;
                    var matchKJHaoMa = string.Empty;
                    for (var i = 1; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Insert(6, "0");
                        ems = tds[2].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 8) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4},{5},{6},{7}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim(), ems[5].InnerText.Trim(), ems[6].InnerText.Trim(), ems[7].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(HUNKL10FJob), string.Format("【{0}】通过备用站点抓取今日最新开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 通过备用站点抓取昨日开奖列表
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromBackUrl()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.BackUrl + "?action=chart&date=yesterday&id=1003&async=true");
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    var obj = JsonConvert.DeserializeObject<JObject>(HtmlResource);
                    if (obj != null && obj["data"] != null)
                    {
                        var matchQiHao = string.Empty;
                        var matchKJHaoMa = string.Empty;
                        JArray openCodeList = null;
                        foreach (var item in obj["data"])
                        {
                            matchQiHao = item["dateNumber"].ToString();
                            openCodeList = (JArray)item["list"];
                            matchKJHaoMa = string.Format("{0},{1},{2},{3},{4},{5},{6},{7}", openCodeList[0].ToString(), openCodeList[1].ToString(), openCodeList[2].ToString(), openCodeList[3].ToString(), openCodeList[4].ToString(), openCodeList[5].ToString(), openCodeList[6].ToString(), openCodeList[7].ToString());
                            if (!result.ContainsKey(matchQiHao))
                                result.Add(matchQiHao, matchKJHaoMa);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(HUNKL10FJob), string.Format("【{0}】通过备用站点抓取昨日开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        /// <summary>
        /// 将此彩种指定期号和开奖号码保存至数据库
        /// </summary>
        /// <param name="QiHao">期号</param>
        /// <param name="OpenCode">开奖号码(形如01,02,03,04,05,06,07,08)</param>
        /// <param name="IsYesterdayRecord">是否是保存昨天的记录</param>
        /// <returns></returns>
        private bool SaveRecord(string QiHao, string OpenCode, bool IsYesterdayRecord)
        {
            if (!string.IsNullOrWhiteSpace(QiHao) && !string.IsNullOrWhiteSpace(OpenCode))
            {
                OpenCode8Model model = new OpenCode8Model();

                string[] arr = QiHao.Split("|".ToCharArray());

                //model.Term = Convert.ToInt64(QiHao);
                model.Term = Convert.ToInt64(arr[0]);
                var haoMaArray = OpenCode.Split(new char[] { ',' });
                model.OpenCode1 = Convert.ToInt32(haoMaArray[0]);
                model.OpenCode2 = Convert.ToInt32(haoMaArray[1]);
                model.OpenCode3 = Convert.ToInt32(haoMaArray[2]);
                model.OpenCode4 = Convert.ToInt32(haoMaArray[3]);
                model.OpenCode5 = Convert.ToInt32(haoMaArray[4]);
                model.OpenCode6 = Convert.ToInt32(haoMaArray[5]);
                model.OpenCode7 = Convert.ToInt32(haoMaArray[6]);
                model.OpenCode8 = Convert.ToInt32(haoMaArray[7]);
                if (IsYesterdayRecord)
                    model.OpenTime = CommonHelper.GenerateYesterdayOpenTime(Config, QiHao);
                else
                    model.OpenTime = Convert.ToDateTime(arr[1]);//CommonHelper.GenerateTodayOpenTime(Config, QiHao);//
                return services.AddOpen8Code(currentLottery, model);
            }
            return false;
        }

        #region Attribute
        /// <summary>
        /// 配置信息
        /// </summary>
        private SCCConfig Config = null;
        /// <summary>
        /// 当天抓取的最新一期期号
        /// </summary>
        private string LatestQiHao = null;
        /// <summary>
        /// 当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;
        /// <summary>
        /// 日志对象
        /// </summary>
        private LogHelper log = null;
        /// <summary>
        /// 数据服务
        /// </summary>
        private IOpen8Code services = null;
        /// <summary>
        /// 当前彩种
        /// </summary>
        private SCCLottery currentLottery
        {
            get
            {
                return SCCLottery.HuNanKL10F;
            }
        }
        /// <summary>
        /// 邮件接口
        /// </summary>
        private IEmail email = null;
        #endregion
    }
}
