﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Quartz;
using HtmlAgilityPack;
using SCC.Common;
using SCC.Models;
using SCC.Interface;

namespace SCC.Crawler.GP
{
    /// <summary>
    /// 数据爬取类
    /// 山东快乐扑克3
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class SDKLPK3Job : IJob
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        public SDKLPK3Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IOpen3Code>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        /// 作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            //预设节假日不开奖
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"))) return;
            LatestQiHao = context.JobDetail.JobDataMap.GetString("LatestQiHao");
            try
            {
                //服务启动时配置初始数据
                if (string.IsNullOrEmpty(LatestQiHao))
                {
                    var lastItem = services.GetLastItem(currentLottery);
                    if (lastItem != null)
                    {
                        LatestQiHao = lastItem.Term.ToString();
                    }
                }
                //第一次启动服务或最新期号为昨天的开奖期号，则自检昨天开奖数据是否抓取完毕(否则插入邮件数据)，并重置当天期号和失败列表
                if (string.IsNullOrEmpty(LatestQiHao) || !LatestQiHao.StartsWith(CommonHelper.SCCSysDateTime.ToString("yyMMdd")))
                {
                    CheckingYesterdayTheLotteryData();
                    LatestQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(0);
                }
                //当最新期号不符合当天总期数，执行当天作业
                if (Convert.ToInt32(LatestQiHao.Substring(6)) != Config.TimesPerDay)
                {
                    DoTodayJobByMainUrl();
                    DoTodayJobByBackUrl();
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(SDKLPK3Job), string.Format("【{0}】抓取时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            //保存最新期号和失败期号列表
            context.JobDetail.JobDataMap["LatestQiHao"] = LatestQiHao;
        }

        /// <summary>
        /// 自检昨天开奖数据
        /// </summary>
        private void CheckingYesterdayTheLotteryData()
        {
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyyMMdd"))) return;//如果昨日设定不开奖则不自检昨日开奖数据
            //从数据库中获取昨天数据抓取失败列表
            FailedQiHaoList = services.GetYesterdayFailQQList(currentLottery, Config.TimesPerDay);
            if (FailedQiHaoList.Count > 0)
            {
                DoYesterdayFailedListByMainUrl();
                DoYesterdayFailedListByBackUrl();
                foreach (var fQiHao in FailedQiHaoList)
                {
                    //将抓取失败数据推送至邮件列表，待邮件服务发送至配置管理员的邮箱中
                    if (email.AddEmail(Config.Area + Config.LotteryName, fQiHao, CommonHelper.GenerateYesterdayOpenTime(Config, fQiHao)))
                        log.Error(typeof(SDKLPK3Job), CommonHelper.GetJobLogError(Config, fQiHao));
                }
            }
        }

        /// <summary>
        /// 通过主站点抓取开奖数据
        /// (网易163)
        /// </summary>
        private void DoTodayJobByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl))
            {
                var OpenList = GetTodayOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.Last().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                string getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(SDKLPK3Job), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过主站抓取错误期号列表中每一个期号
        /// (网易163)
        /// </summary>
        private void DoYesterdayFailedListByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(SDKLPK3Job), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }

        /// <summary>
        /// 通过主站点抓取今日最新开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromMainUrl()
        {
            var requestUrl = string.Format(Config.MainUrl, 1);
            return GetOpenListFromMainUrl(requestUrl);
        }
        /// <summary>
        /// 通过主站点抓取昨日开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromMainUrl()
        {
            var requestUrl = string.Format(Config.MainUrl, 2);
            return GetOpenListFromMainUrl(requestUrl);
        }
        /// <summary>
        /// 抓取主站点开奖数据
        /// </summary>
        /// <param name="url">主站点</param>
        /// <returns></returns>
        private Dictionary<string, string> GetOpenListFromMainUrl(string url)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(url);
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var tbody = doc.GetElementbyId("cpdata");
                    if (tbody == null) return result;
                    var trs = tbody.ChildNodes.Where(node => node.Name == "tr").ToList();
                    if (trs.Count < 1) return result;
                    string matchQiHao = string.Empty;
                    string matchKJHaoMa = string.Empty;
                    string tdClass = string.Empty;
                    string opencode1 = string.Empty;
                    string opencode2 = string.Empty;
                    string opencode3 = string.Empty;
                    for (var i = 0; i < trs.Count; i++)
                    {
                        var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();
                        if (tds.Count < 5) continue;
                        matchQiHao = tds[0].InnerText.Trim();
                        tdClass = (tds[2].ChildNodes.Count >= 2 && tds[2].ChildNodes[1].Attributes["class"] != null) ? tds[2].ChildNodes[1].Attributes["class"].Value : string.Empty;
                        opencode1 = GetMainPuKeString(tds[2].InnerText.Trim(), tdClass);
                        tdClass = (tds[3].ChildNodes.Count >= 2 && tds[3].ChildNodes[1].Attributes["class"] != null) ? tds[3].ChildNodes[1].Attributes["class"].Value : string.Empty;
                        opencode2 = GetMainPuKeString(tds[3].InnerText.Trim(), tdClass);
                        tdClass = (tds[4].ChildNodes.Count >= 2 && tds[4].ChildNodes[1].Attributes["class"] != null) ? tds[4].ChildNodes[1].Attributes["class"].Value : string.Empty;
                        opencode3 = GetMainPuKeString(tds[4].InnerText.Trim(), tdClass);
                        matchKJHaoMa = string.Format("{0},{1},{2}", opencode1, opencode2, opencode3);
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(SDKLPK3Job), string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 将花色和号码转化为开奖号码
        /// 1为方片，2为红心，3为梅花，4为黑桃
        /// A为1，J为11，Q为12，K为13
        /// </summary>
        /// <param name="KJHaoMa">号码</param>
        /// <param name="ClassName">花色类名称</param>
        /// <returns></returns>
        private string GetMainPuKeString(string KJHaoMa, string ClassName)
        {
            string result = string.Empty;
            switch (KJHaoMa)
            {
                case "A":
                    result = "01";
                    break;
                case "J":
                    result = "11";
                    break;
                case "Q":
                    result = "12";
                    break;
                case "K":
                    result = "13";
                    break;
                default:
                    result = KJHaoMa.PadLeft(2, '0');
                    break;
            }
            if (ClassName.Contains("fangpian"))
                return "1" + result;
            if (ClassName.Contains("hongtao"))
                return "2" + result;
            if (ClassName.Contains("meihua"))
                return "3" + result;
            return "4" + result;//heitao
        }

        /// <summary>
        /// 通过备用站点抓取开奖数据
        /// (新浪爱彩)
        /// </summary>
        private void DoTodayJobByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl))
            {
                var OpenList = GetTodayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.Last().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                var getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(SDKLPK3Job), CommonHelper.GetJobBackLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过备用地址抓取错误期号列表中每一个期号
        /// (新浪爱彩)
        /// </summary>
        private void DoYesterdayFailedListByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (var failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(SDKLPK3Job), CommonHelper.GetJobBackLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }

        /// <summary>
        /// 通过备用站点抓取今日最新开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromBackUrl()
        {
            return GetOpenListFromBackUrl(Config.BackUrl, "sDate=&eDate=&q=today&sortTag=up");
        }
        /// <summary>
        /// 通过备用站点抓取昨日开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromBackUrl()
        {
            return GetOpenListFromBackUrl(Config.BackUrl, "sDate=&eDate=&q=yesterday&sortTag=up");
        }
        /// <summary>
        /// 抓取备用站点开奖数据
        /// </summary>
        /// <param name="url">备用站点</param>
        /// <param name="postdata">发送数据</param>
        /// <returns></returns>
        private Dictionary<string, string> GetOpenListFromBackUrl(string url, string postdata)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(url, "POST", postdata);
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var table = doc.GetElementbyId("chartsTable");
                    if (table == null) return result;
                    var trs = table.SelectSingleNode("tbody").ChildNodes.Where(node => node.Name == "tr").ToList();
                    if (trs.Count < 1) return result;
                    string matchQiHao = string.Empty;
                    string matchKJHaoMa = string.Empty;
                    for (var i = 0; i < trs.Count; i++)
                    {
                        var trstyle = trs[i].Attributes["style"];
                        if (trstyle != null && trstyle.Value == "display:none")
                        {
                            continue;
                        }
                        var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();
                        if (tds.Count < 5) continue;
                        matchQiHao = CommonHelper.SCCSysDateTime.ToString("yy") + tds[0].InnerText.Trim();
                        matchKJHaoMa = string.Format("{0},{1},{2}", GetBackPuKeString(tds[2].InnerText.Trim()), GetBackPuKeString(tds[3].InnerText.Trim()), GetBackPuKeString(tds[4].InnerText.Trim()));
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(SDKLPK3Job), string.Format("【{0}】通过备用站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 将备用站点的快乐扑克3号码转化成所需开奖号码
        /// </summary>
        /// <param name="KJHaoMa">开奖号码</param>
        /// <returns></returns>
        private string GetBackPuKeString(string KJHaoMa)
        {
            if (string.IsNullOrWhiteSpace(KJHaoMa)) return string.Empty;
            if (!"♦♥♣♠".Contains(KJHaoMa.Substring(0, 1))) return string.Empty;
            var haoma = KJHaoMa.Substring(1);
            switch (haoma)
            {
                case "A":
                    haoma = "01";
                    break;
                case "J":
                    haoma = "11";
                    break;
                case "Q":
                    haoma = "12";
                    break;
                case "K":
                    haoma = "13";
                    break;
                default:
                    haoma = haoma.PadLeft(2, '0');
                    break;
            }
            if (KJHaoMa.Substring(0, 1) == "♦")
                return "1" + haoma;
            if (KJHaoMa.Substring(0, 1) == "♥")
                return "2" + haoma;
            if (KJHaoMa.Substring(0, 1) == "♣")
                return "3" + haoma;
            return "4" + haoma;
        }


        /// <summary>
        /// 将此彩种指定期号和开奖号码保存至数据库
        /// </summary>
        /// <param name="QiHao">期号</param>
        /// <param name="OpenCode">开奖号码(形如01,02,03,04,05)</param>
        /// <param name="IsYesterdayRecord">是否是保存昨天的记录</param>
        /// <returns></returns>
        private bool SaveRecord(string QiHao, string OpenCode, bool IsYesterdayRecord)
        {
            if (!string.IsNullOrWhiteSpace(QiHao) && !string.IsNullOrWhiteSpace(OpenCode))
            {
                OpenCode3Model model = new OpenCode3Model();
                model.Term = Convert.ToInt64(QiHao);
                var haoMaArray = OpenCode.Split(new char[] { ',' });
                model.OpenCode1 = Convert.ToInt32(haoMaArray[0]);
                model.OpenCode2 = Convert.ToInt32(haoMaArray[1]);
                model.OpenCode3 = Convert.ToInt32(haoMaArray[2]);
                if (IsYesterdayRecord)
                    model.OpenTime = CommonHelper.GenerateYesterdayOpenTime(Config, QiHao);
                else
                    model.OpenTime = CommonHelper.GenerateTodayOpenTime(Config, QiHao);
                return services.AddOpen3Code(currentLottery, model);
            }
            return false;
        }

        #region Attribute
        /// <summary>
        /// 配置信息
        /// </summary>
        private SCCConfig Config = null;
        /// <summary>
        /// 当天抓取的最新一期期号
        /// </summary>
        private string LatestQiHao = null;
        /// <summary>
        /// 当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;
        /// <summary>
        /// 日志对象
        /// </summary>
        private LogHelper log = null;
        /// <summary>
        /// 数据服务
        /// </summary>
        private IOpen3Code services = null;
        /// <summary>
        /// 当前彩种
        /// </summary>
        private SCCLottery currentLottery
        {
            get
            {
                return SCCLottery.ShanDongKLPK3;
            }
        }
        /// <summary>
        /// 邮件接口
        /// </summary>
        private IEmail email = null;
        #endregion
    }
}
